
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "EKG.h"

// Konstruktor
EKG::EKG( int argc, char *argv[] ) : Pipeend(Freq),
    g(NULL), h(false), p(NULL), r(false), t(0), w(0), H(-1), L(-1), N(NULL), 
    P(false), S(false), T(50), V(20), device(NULL)
{
  // Kommandozeile lesen.
  int opt;
  while ( -1 != ( opt = getopt( argc, argv, "g:hp:rt:w:H::L::N:PST:V:" ) ) )
    switch ( opt )
    {
      case 'g' : g = optarg;
                 break;
      case 'h' : h = true;
                 break;
      case 'p' : p = optarg;
                 break;
      case 'r' : r = true;
                 break;
      case 't' : t = atoi( optarg );
                 break;
      case 'w' : w = atoi( optarg );
                 break;
      case 'H' : if ( optarg ) H = atof( optarg );
                 else H = 0.3;
                 break;
      case 'L' : if ( optarg ) L = atof( optarg );
                 else L = 10;
                 break;
      case 'N' : N = optarg;
                 break;
      case 'P' : P = true;
                 break;
      case 'S' : S = true;
                 break;
      case 'T' : T = atof( optarg );
                 break;
      case 'V' : V = atof( optarg );
                 break;
    }
  device = argv[ optind ];
  // Auf Richtigkeit prfen. Sehr grob...
  if ( !g && !p && !r && !h )
    throw string( "What shell I do?" );
  if ( p && t )
    throw string( "p and t are exclusive." );
  if ( !h && optind >= argc )
    throw string( "Which serial port?" );
  // Signalflu
  Source.connectTo( &Hfilter );
  Hfilter.connectTo( &Lfilter );
  Lfilter.connectTo( &Freq );
  // Freq ist Pipeend
}



// Warten
void EKG::wait()
{
  cerr << w << " seconds to wait..." << endl;
  sleep( w );
}



// Programmlauf
void EKG::run()
{
  if ( h ) { help(); return; }
  wait();
  // Gerte Einstellen
  Source.serialOpen( device );
  if ( r ) { Source.raw(); return; }
  Hfilter.set_enable( H > 0 );
  if ( H > 0 ) Hfilter.setHTimeConstant( H );
  Lfilter.set_enable( L > 0 );
  if ( L > 0 ) Lfilter.setLFreq( L );
  Freq.setFreq( P );
  Freq.setBeep( S );
  if ( p ) program_ps();
  else program_data();
}


// Programm PS
void EKG::program_ps()
{
  EKGbuffer Buffer( EKGplotter::timesteps_on_paper( T ) );
  Pipeend.connectTo( & Buffer );
  int recTime = static_cast<int> // Messung ganze Sek
                ( ceil( EKG_TimeStep * EKGplotter::timesteps_on_paper( T ) ) );
  cerr << "Recording " << recTime << " seconds..." << endl;
  Source.timer( recTime );
  EKGplotter Plotter( Buffer );
  Plotter.ts = T;
  Plotter.vs = V;
  if ( N ) Plotter.name = N;
  ofstream out( p );
  Plotter.toPostscript( out );
  out.close();
  if ( g )
  {
    out.open( g );
    Plotter.toRaw( out );
    out.close();
  }
}


// Programm Data
void EKG::program_data()
{
  if ( t ) // endliche Messung
  {
    EKGbuffer Buffer( t / EKG_TimeStep );
    Pipeend.connectTo( & Buffer );
    cerr << "Recording " << t << " seconds..." << endl;
    Source.timer( t );
    EKGplotter Plotter( Buffer );
    if ( N ) Plotter.name = N;
    ofstream out( g );
    Plotter.toRaw( out );
    out.close();
  }
  else // unendliche Messung
  {
    EKGlogger Logger;
    Pipeend.connectTo( & Logger );
    string name;
    if ( N ) name = N;
    ofstream out( g );
    cerr << "Start recording. Press RETURN to stop." << endl;
    Logger.start( name, out );
    Source.start();
    // Auf Taste warten
    while ( cin.get() == EOF ) sleep( 200 );
    Source.stop();
    Logger.stop();
    out.close();
    cerr << "Recording finished." << endl;
  }
}


// Hilfe Ausgeben
void EKG::help()
{
  cout << "" << endl;
  cout << "ekg (C) 2003 Lars Stollenwerk" << endl;
  cout << "This program comes with ABSOLUTELY NO WARRANTY." << endl;
  cout << "It is distributed under the terms of GPL." << endl;
  cout << "" << endl;
  cout << "Print raw data from hardware. Use to calibrate." << endl;
  cout << "  ekg -r [-w time] device" << endl;
  cout << "" << endl;
  cout << "Write EKG as postscript. Optionally write unscaled data to file (-g)." << endl;
  cout << "Timescale (-T) defaults to 50 mm/s and voltagescale (-V) defaults to 20 mm/mV." << endl;
  cout << "Recording time is determined by timescale (and paperlength)." << endl;
  cout << "  ekg -p file [-g file] [-T ts] [-V vs] [options] device" << endl;
  cout << "" << endl;
  cout << "Write unscaled EKG to file. Recording ends on RETURN or after -t seconds." << endl;
  cout << "  ekg -g file [-t time] [options] device" << endl;
  cout << "" << endl;
  cout << "Options:" << endl;
  cout << "  -w time   Wait time seconds before recording." << endl;
  cout << "  -L[freq]  Use lowpass [and set frequency, default = 10 Hz]." << endl;
  cout << "  -H[time]  Use highpass [and set time constant, default = 0.3 s]." << endl;
  cout << "  -P        Print pulse in bpm on stderr." << endl;
  cout << "  -S        Beep on heartbeat. (Sound)" << endl;
  cout << "  -N name   Set patinents name (and birthday)." << endl;
}


// Hauptprogramm.
int main (int argc, char *argv[])
{
  try
  {
    EKG ekg( argc, argv );
    ekg.run();
  }
  catch ( string s )
  { 
    cerr << s << endl; 
    cerr << "'-h' for help." << endl;
    exit(1); 
  }
  cerr << endl << "READY." << endl;
}

