
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef EKGSOURCE_H
#define EKGSOURCE_H

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>
#include <unistd.h>
#include <pthread.h>

#include <string>
#include <sstream>
#include <iostream> 

#include "EKGdevice.h"
#include "calibrate.h"

/**
 * Diese Klasse verwaltet die serielle Schnittstelle. 
 * <p>
 * Die Schnittstelle wird mit serialOpen() geffnet. Mit connectTo() wird
 * ein Empgnger der Daten bestimmt. Mit start() und stop() kann ein Sender
 * ein- und ausgeschaltet werden; dabei luft der Sender in einem anderen
 * Thread. Mit timer() wird der Sender fr eine bestimmte Zeitspanne
 * eingeschaltet. 
 * <p>
 * <b>Achtung!</b> Die von EKGsource an den Empfnger geandten Daten sind
 * nur solange gltig, wie die Instanz von EKGsource existiert. Sollen sie
 * spter weiterverarbeitet werden, so mssen sie kopiert werden!
 *
 * @author Lars Stollenwerk, Oktober 2002
 */
class EKGsource
{
public:
  EKGsource() : sending(false), nextDevice(NULL) {}
  ~EKGsource();
  ///  ffnet Serielle Schnittstelle.
  void serialOpen( const char * pathname );
  /**
   * Startet den Sender in einem eigenen Thread. 
   */
  void start();
  /**
   * Hlt den Sender an und lscht den Thread.
   */
  void stop();
  /**
   * Sendet fr eine feste Dauer.
   * @param t Sendezeit in Sekunden
   */
  void timer( int t );
  /**
   * Nur zu Debugzwecken: gibt Rohdaten auf stdout aus.
   * Diese Funktion arbeitet am Signalweg ber EKGdevices und Datenkrbchen
   * vorbei und gibt die Rohdaten direkt auf stdout aus.
   */
  void raw();
  /** 
   * Anschlu eines nachfolgenden Gertes.
   * @param Zeiger auf neues nachfolgendes Gert.
  */
  void connectTo( EKGdevice * d );
private:
  // ----------------------------------------------------------------------
  // Serielle Schnittstelle
  // ----------------------------------------------------------------------

  /**
   * File Handler fr die serielle Schnittstelle
   */
  int SerialPort;
  /**
   * IO Einstellungen der seriellen Schnittstelle
   */
  termios SerialIOs;
  /**
   * Liest genau ein Byte vom seriellen Port. Wartet ohne Timeout auf
   * Daten.
   */
  int readByte();  

  // ----------------------------------------------------------------------
  // Datenstrom
  // ----------------------------------------------------------------------

  /**
   * Der Datenstrom enthlt drei Mewerte, gefolgt von einer groen
   * konstanten Zahl. Diese Methode synchronisiert den Datenstrom so, da
   * die grte Zahl immer am Ende eines Wertequartetts steht.
   */
  void synchronise();
  /**
   * Verwirft alle empfangenen aber noch nicht gelesenen Daten. Die
   * Synchronisation wird dadurch zerstrt, flush() sollte also immer von
   * synchronise() gefolgt werden.
   */
  void flush();
  /**
   * Erzeugt aus Rohdaten Spannungstripletts. Die Werte werden im
   * ,,Datenkrbchen'' zum Versand abgelegt. Eichkurven der Meverstrker
   * werden in calibrate.h festgelegt.
   */
  void fillBasket();


  // ----------------------------------------------------------------------
  // Sender
  // ----------------------------------------------------------------------

  /**
   * Thread, in dem der Sender luft.
   */
  pthread_t senderID;
  /**
   * Flag: Sender ist an.
   */
  bool sending;
  /**
   * Speicherplatz, der an den Empfnger weitergereicht wird.
   */
  float dataBasket[3];
  /**
   * Zeiger auf nachfolgendes Gert.
   */
  EKGdevice * nextDevice;
  /**
   * Sender, der im Thread luft. Die Methode enthlt eine Endlosschleife.
   * Der Sender wird in stop() durch pthread_cancel() angehalten.
   *
   * @param me Ziger auf EKGsource-Instanz.
   * @return Dummy fr pthread.
   */
  static void * Sender( void * Me );
};

#endif

